# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module basicwidgets 
##   Widgets spcifiques  pycc, tendant les widgets de Tkinter
## ----------------------------------------------------------------------

from Tkinter import *
from ScrolledText import ScrolledText
import tkSimpleDialog
import tkFileDialog
import tkColorChooser
import Pmw
import os.path

import logging
log = logging.getLogger('pycc')

# Fontes spciales os-dpendantes
import os
if os.name == 'posix':
  BBUT_FONT = 'Times 4'
else:
  BBUT_FONT = 'Times 7'


class SplashWindow(object):
  
  def __init__(self, parent, title, text, anchor=SW):
    self._splash = Frame(parent, relief=RAISED, borderwidth=2)
    self.parent = parent
    self.title = Label(self._splash, text=title, cursor="arrow", background="#F0F0FF", font="Helvetica 9 bold")
    self.title.bind("<ButtonRelease>", self.destroy)
    self.title.pack(fill=X)
    self.text = Label(self._splash, text=text, cursor="arrow", background="#FFEAA0")
    self.text.bind("<ButtonRelease>", self.destroy)
    self.text.pack(fill=BOTH, expand=1)
    self.anchor = anchor
    self.display()
  
  def destroy(self, event=None):
    self._splash.place_forget()
  
  def setText(self, text):
    self.text.config(text=text)
  
  def display(self):
    if self.anchor == SE:
      xpos = 1.0
    else:
      xpos = 0.0
    self._splash.place(in_=self.parent, relx=xpos, rely=1.0, anchor=self.anchor)
  

class Led(Frame):
  
  def __init__(self, master, label="", **kwargs):
    self.root = master
    Frame.__init__(self, master)
    self._lbl = Label(self, text=label)
    self._lbl.pack(side=LEFT)
    self._led = Label(self, text="", relief=GROOVE, font="Helvetica 7", width=4, **kwargs)
    self._led.pack(side=LEFT)
    self.system_color = self.cget('background')
    self.message = ""
  
  def setState(self, color, message=''):
    self._led.config(background=color)
    if message:
      self.message = message
  
  def clear(self):
    self._led.config(background=self.system_color)
  
  def setText(self, msg=''):
    if msg == '0':
      txt = ''
    else:
      txt = msg
    self._led.config(text=txt)
  
  def getText(self):
    return self._led.cget("text")
  
  def incr(self, cpt=1):
    try:
      cur_val = int(self.getText())
    except:
      cur_val = 0
    self.setText(str(cur_val+cpt))
  
  def setLabel(self, lbl=''):
    self._lbl.config(text=lbl)
  
  def showMsg(self, parent, title, anchor=SW):
    SplashWindow(parent, title, self.message, anchor)
  

class TextButton(Label):
  def __init__(self, parent, text, command=lambda x=None:None, **kw):
    self.text = text
    self.command = command
    Label.__init__(self, parent, text=self.text, width=4, font="Helvetica 8", relief=FLAT, border=1)
    self.bind("<ButtonRelease-1>", lambda ev, s=self: s.action())
    self.bind("<Button-1>", lambda ev, s=self: s.config(relief=SUNKEN))
    self.bind("<Enter>", lambda ev, s=self: s.config(relief=RAISED))
    self.bind("<Leave>", lambda ev, s=self: s.config(relief=FLAT))
  
  def action(self):
    self.command()

class IconButton(Button):
  def __init__(self, parent, icon1, icon2=None, command=lambda x=None:None, **kw):
    self.icon1 = PhotoImage(icon1)
    if icon2:
      self.icon2 = PhotoImage(icon2)
    else:
      self.icon2 = None
    Button.__init__(self, parent, command=self.action, icon=self.icon1, relief=FLAT, border=1, width=20, **kw)
    self.pressed = 0
    self.command = command
    self.bind("<Button-1>", lambda ev, s=self: s.config(relief=SUNKEN))
    self.bind("<Enter>", lambda ev, s=self: s.config(relief=RAISED))
    self.bind("<Leave>", lambda ev, s=self: s.config(relief=FLAT))
  
  def action(self):
    self.pressed = not self.pressed
    if self.pressed:
      if self.icon2:
        self.config(icon=self.icon2)
      self.config(relief=SUNKEN)
    self.command()
  
class ToolBar(Frame):
  
  def __init__(self, parent, **kw):
    Frame.__init__(self, parent, **kw)
    self.buttons = {}
  
  def addButton(self, name, text='', icon=None, command=lambda x=None:None):
    if icon is not None:
      self.buttons[name] = IconButton(self, icon1=icon, command=command)
    else:
      self.buttons[name] = TextButton(self, text=text, command=command)
    self.buttons[name].pack(side=LEFT, anchor=W, padx=1, fill=Y, expand=1)
  

class TreeStruct(object):
  def __init__(self, root, childs=[]):
    self.root = root
    self.childs = childs
    self.expanded = 1
  
  def path(self, id, res=[]):
    if self.root.id == id:
      return res
    if self.childs:
      for subtree in self.childs:
        buf = subtree.path(id, res+[self])
        if buf is not None:
          return buf
    else:
      return None
  
  def findAll(self, id):
    res = []
    if self.root.id == id:
      res = [self]
    for subtree in self.childs:
      res += subtree.findAll(id)
    return res
  
  def collapseAll(self):
    self.expanded = 0
    for subtree in self.childs:
      subtree.collapseAll()
  
  def expandAll(self):
    self.expanded = 1
    for subtree in self.childs:
      subtree.expandAll()
  
  def addChild(self, id, node):
    iroot = self.findAll(id)
    if not iroot: return
    iroot[0].childs.append(node)
  
  def removeChild(self, id, childid):
    iroot = self.findAll(id)
    if not iroot: return
    newchilds = [c for c in iroot[0].childs if c.root.id != childid]
    iroot[0].childs = newchilds
  
  def length(self):
    res = 1
    for subtree in self.childs:
      res += subtree.length()
    return res
  
  def flatten(self):
    res = [self.root]
    for subtree in self.childs:
      res += subtree.flatten()
    return res
  
  def abslength(self):
    res = []
    for item in self.flatten():
      if item not in res:
        res.append(item)
    return len(res)
  
  def __str__(self):
    res = "%s [ " % self.root.id
    for c in self.childs:
      res += "%s," % c
    return res[:-1] + " ]"

class Tree(ScrolledText):
  
  def __init__(self, parent, tree, ini_val=None, hcolor="#FCFCDA", hborder=1, hrelief=RAISED, **kwargs):
    ScrolledText.__init__(self, parent, spacing1=hborder, spacing3=hborder, font="Helvetica 9")
    self.config(**kwargs)
    self.root = parent
    self.hcolor = hcolor
    self.hborder = hborder
    self.hrelief = hrelief
    self.tree = tree
    self.ini_val = ini_val
    self.selected = None
    self.config(cursor="arrow", state=DISABLED, selectforeground=self.cget('foreground'), selectbackground=self.cget('background'))
    self.bind("<Enter>", lambda e, w=self: w.focus())
    self.bind("<Up>", self.select_prev)
    self.bind("<Down>", self.select_next)
    self.bind("<Left>", lambda e, s=self: s.collapse(s.selected))
    self.bind("<Right>", lambda e, s=self: s.expand(s.selected))
    self.bind("<Return>", self.toggle_current)
    self.bind("<1>", self.click)
    self.bind("<Double-1>", lambda e, s=self: s.click(e, 1))
    self.images = { \
      'plus':  PhotoImage(
                  data='R0lGODdhCQAJAPEAAAAAAH9/f////wAAACwAAAAACQAJAAAC' \
                  'EYyPoivG614LAlg7ZZbxoR8UADs='),
      'minus': PhotoImage(
                  data='R0lGODdhCQAJAPEAAAAAAH9/f////wAAACwAAAAACQAJAAAC' \
                  'FIyPoiu2sJyCyoF7W3hxz850CFIA\nADs='),
      'leaf':  PhotoImage(
                  data='R0lGODlhCwAOAJEAAAAAAICAgP///8DAwCH5BAEAAAMALAAA' \
                  'AAALAA4AAAIphA+jA+JuVgtUtMQePJlWCgSN9oSTV5lkKQpo2q5W+' \
                  'wbzuJrIHgw1WgAAOw==')
    }
    self.bind_select(lambda w=self: None)
    # print "### init Tree:", self.tree.root.id, type(self.tree.root.id)
    self.reset()
  
  def bind_select(self, callback):
    self.select_callback = callback
  
  def reset(self):
    self._disp_tree()
  
  def add_child(self, id, child):
    self.tree.addChild(id, child)
    self.reset()
  
  def remove_child(self, id, childid):
    self.tree.removeChild(id, childid)
    self.reset()
  
  def select_prev(self, event=None):
    # print "### select_prev"
    if self.selected is None:
      tags = self.tag_names(1.0)
    else:
      tags = self.tag_names("id_%s.first-2c" % self.selected)
    for tag in tags:
      if tag[:3] == "id_": self.select(tag[3:])
  
  def select_next(self, event=None):
    # print "### select_next"
    if self.selected is None:
      tags = self.tag_names(1.0)
    else:
      tags = self.tag_names("id_%s.last+2c" % self.selected)
    for tag in tags:
      if tag[:3] == "id_": self.select(tag[3:])
  
  def select(self, item):
    # print "### select:", item, type(item)
    if item not in [node.id for node in self.tree.flatten()]:
      return
    if self.selected is not None:
      self.tag_config("id_%s" % self.selected, background=self.cget("background"), relief=FLAT)
    to_redraw = 0
    for node in self.tree.path(item):
      if not node.expanded: to_redraw = 1
      node.expanded = 1
    if to_redraw: self._disp_tree()
    self.tag_config("id_%s" % item, background=self.hcolor, borderwidth=self.hborder, relief=self.hrelief)
    self.selected = item
    start, end = self.tag_ranges("id_%s" % item)
    self.see(start)
    self.select_callback()
  
  def click(self, event, double=0):
    tags = self.tag_names("@%d,%d" % (event.x, event.y))
    # print "### tags =", tags
    cur_id = None
    to_toggle = double
    for t in tags:
      if t == "foldsign":
        to_toggle = 1
      elif t[:3] == 'id_':
        cur_id = t[3:]
    if cur_id is not None:
      if to_toggle: self.toggle(cur_id)
      self.select(cur_id)
  
  def toggle_current(self, event=None):
    if self.selected is None: return
    self.toggle(self.selected)
  
  def toggle(self, id):
    # print "### toggle:", id, type(id)
    for node in self.tree.findAll(id):
      node.expanded = not node.expanded
      # print "### node:", node.root.id, "expanded=", node.expanded
    self._disp_tree()
  
  def expand(self, id):
    for node in self.tree.findAll(id):
      node.expanded = 1
    self._disp_tree()
  
  def collapse(self, id):
    for node in self.tree.findAll(id):
      node.expanded = 0
    self._disp_tree()
  
  def insertItem(self, item):
    # A surcharger ventuellement (comment insrer un item dans l'arbre)
    self.insert(END, item.label+"\n")
  
  def _disp_tree(self):
    self.config(state=NORMAL)
    self.delete(0.0, END)
    self._recurse_disp(self.tree)
    self.config(state=DISABLED)
  
  def _recurse_disp(self, tree, blk=""):
    if len(tree.childs) > 0:
      if tree.expanded:
        sign = self.images['plus']
      else:
        sign = self.images['minus']
    else:
      sign = None # self.images['leaf']
    self.mark_set(INSERT, END)
    index = self.index(INSERT)
    self.insert(END, blk)
    if sign is not None:
      self.image_create(END, image=sign, align=CENTER)
      self.insert(END, " ")
      self.tag_add("foldsign", "end-3c", "end-1c")
    if hasattr(tree.root, 'image') and tree.root.image is not None:
      self.image_create(END, image=tree.root.image, align=CENTER)
      self.insert(END, " ")
    #self.insert(END, tree.root.label+"\n")
    self.insertItem(tree.root)
    self.tag_add("id_%s" % tree.root.id, index, self.index(INSERT))
    if tree.expanded:
      for child in tree.childs:
        self._recurse_disp(child, blk+"      ")

from urllib import quote, unquote  # pour viter les pb d'espaces
class ListBox(ScrolledText):
  
  def __init__(self, parent, items={}, ini_val=None, hcolor="#FCFCDA", hborder=1, hrelief=RAISED, **kwargs):
    ScrolledText.__init__(self, parent, spacing1=hborder, spacing3=hborder, font="Helvetica 9")
    self.config(**kwargs)
    self.hcolor = hcolor
    self.hborder = hborder
    self.hrelief = hrelief
    self.items = {}
    if type(items) != type({}):
      tmp_items = {}
      for i in items: tmp_items[i] = i
      items = tmp_items
    for i in items.keys():
      self.add(i, items[i])
    self.ini_val = ini_val
    self.config(cursor="arrow", state=DISABLED)
    self.bind("<Enter>", lambda e, w=self: w.focus())
    self.bind("<Up>", self.select_prev)
    self.bind("<Down>", self.select_next)
    self.bind_select(lambda w=self: None)
    self.selected = None
    self.reset()
  
  def bind_select(self, callback):
    self.select_callback = callback
  
  def add(self, name, label=None):
    if name in self.items.keys():
      return
    if label is None:
      label = name
    try:
      lbl = unicode(label, 'iso-8859-15')
    except:
      lbl = label
    self.config(state=NORMAL)
    tag = quote("%s" % name)
    # print "#####", type(lbl), repr(lbl)
    self.insert(END, " %s\n" % lbl, tag)
    self.tag_bind(tag, "<ButtonRelease-1>", lambda e, w=self, item=name: w.select(item))
    self.config(state=DISABLED)
    self.items[name] = label
  
  def empty(self):
    self.config(state=NORMAL)
    self.delete(0.0, END)
    self.config(state=DISABLED)
    self.items = {}
  
  def remove(self, name):
    if name not in self.items.keys():
      return
    tag = quote("%s" % name)
    start, end = self.tag_ranges(tag)
    self.config(state=NORMAL)
    self.tag_delete(tag)
    self.delete(start, end)
    self.config(state=DISABLED)
    del self.items[name]
    if name == self.selected:
      self.selected = None
  
  def get(self, name=None):
    if name is None:
      if self.selected is not None:
        return self.selected
      else:
        return None
    if name not in self.items.keys():
      return ""
    else:
      return self.items[name]
  
  def set(self, value):
    self.select(value)
  
  def reset(self):
    if self.ini_val != None:
      if self.ini_val in self.items.keys():
        self.select(self.ini_val)
  
  def length(self):
    return len(self.items)
  
  def select_prev(self, event=None):
    if self.selected is None:
      tag = self.tag_names(1.0)
    else:
      tag = self.tag_names("%s.first-2c" % quote("%s" % self.selected))
    if tag:
      self.select(unquote("%s" % tag[0]))
  
  def select_next(self, event=None):
    if self.selected is None:
      tag = self.tag_names(1.0)
    else:
      tag = self.tag_names("%s.last+2c" % quote("%s" % self.selected))
    if tag:
      self.select(unquote("%s" % tag[0]))
  
  def select(self, item):
    if item not in self.items.keys():
      return
    if self.selected != None:
      self.tag_config(quote("%s" % self.selected), background=self.cget("background"), relief=FLAT)
    tag = quote("%s" % item)
    self.tag_config(tag, background=self.hcolor, borderwidth=self.hborder, relief=self.hrelief)
    self.selected = item
    start, end = self.tag_ranges(tag)
    self.see(start)
    self.select_callback()
  
class SortedListBox(ListBox):
  
  def __init__(self, parent, items, **kwargs):
    ListBox.__init__(self, parent, items, **kwargs)
    self.update()
  
  def update(self):
    slist = [ (val, key) for key, val in self.items.items() ]
    slist.sort()
    self.empty()
    for value, key in slist:
      self.add(key, value)
  
  
class OrderedListBox(ListBox):

  def __init__(self, parent, items, **kwargs):
    self.olist = items
    ListBox.__init__(self, parent, {}, **kwargs)
    self.refresh()
  
  def refresh(self):
    self.empty()
    for cpt, i in enumerate(self.olist):
      self.add(cpt, i)
  
  def getAll(self):
    return self.olist
  
  def insertItem(self, item, pos=0):
    for i in range(pos, len(self.olist)):
      self.remove(i)
    self.olist.insert(pos, item)
    for i in range(pos, len(self.olist)):
      self.add(i, self.olist[i])
  
  def deleteItem(self, pos=None):
    if pos is None:
      pos = int(self.selected)
    if pos is None: return
    for i in range(pos, len(self.olist)):
      self.remove(i)
    del self.olist[pos]
    for i in range(pos, len(self.olist)):
      self.add(i, self.olist[i])
  
  def upper(self):
    if self.selected is None: return
    pos = int(self.selected)
    if pos > 0:
      item = self.olist[pos]
      self.deleteItem(pos)
      self.insertItem(item, pos-1)
      self.select(pos-1)
  
  def lower(self):
    if self.selected is None: return
    pos = int(self.selected)
    if pos < len(self.olist)-1:
      item = self.olist[pos]
      self.deleteItem(pos)
      self.insertItem(item, pos+1)
      self.select(pos+1)
  
class OrderedListBoxWithButtons(OrderedListBox):

  def __init__(self, parent, items, editbutton=0, balloon=1, **kwargs):
    self.fr = Frame(parent)
    OrderedListBox.__init__(self, self.fr, items, **kwargs)
    self.balloon = Pmw.Balloon(parent)
    if balloon:
      self.balloon.configure(state='balloon')
    else:
      self.balloon.configure(state='none')
    bbox = Frame(self.fr)
    w = Button(bbox, text="^", width=3, border=1, command=self.upperItem)
    w.pack(side=LEFT, fill=X, expand=1)
    self.balloon.bind(w, "Dplacer vers le haut")
    w = Button(bbox, text="v", width=3, border=1, command=self.lowerItem)
    w.pack(side=LEFT, fill=X, expand=1)
    self.balloon.bind(w, "Dplacer vers le bas")
    w = Button(bbox, text="D", width=3, border=1, command=self.removeItem)
    w.pack(side=LEFT, fill=X, expand=1)
    self.balloon.bind(w, "Supprimer l'lment slectionn")
    w = Button(bbox, text="N", width=3, border=1, command=self.addItem)
    w.pack(side=LEFT, fill=X, expand=1)
    self.balloon.bind(w, "Insrer un nouvel lment")
    if editbutton:
      w = Button(bbox, text="E", width=3, border=1, command=self.editItem)
      w.pack(side=LEFT, fill=X, expand=1)
      self.balloon.bind(w, "Editer l'lment slectionn")
      self.bind("<Double-Button-1>", self.editItem)
    self.pack(fill=BOTH, expand=1)
    bbox.pack(fill=X)
    self.bind_new(lambda x=parent: tkSimpleDialog.askstring("pyCoinCoin", "Nouvel lment", parent=x))
    self.bind_delete(lambda x=None: 1)
    self.bind_edit(lambda x=None: 0)
    self.bind_upper(lambda x=None: None)
    self.bind_lower(lambda x=None: None)
    self.pack = self.fr.pack
  
  def bind_new(self, callback):
    self.new_callback = callback
  
  def bind_delete(self, callback):
    self.del_callback = callback
  
  def bind_edit(self, callback):
    self.edit_callback = callback
  
  def bind_upper(self, callback):
    self.upper_callback = callback
  
  def bind_lower(self, callback):
    self.lower_callback = callback
  
  def lowerItem(self):
    self.lower_callback()
    self.lower()
  
  def upperItem(self):
    self.upper_callback()
    self.upper()
  
  def addItem(self):
    res = self.new_callback()
    if res is None: return
    if self.selected is None:
      pos = 0
    else:
      pos = int(self.selected)
    self.insertItem(res, pos)
    self.select(pos)
  
  def removeItem(self):
    if self.del_callback():
      self.deleteItem()

  def editItem(self, event=None):
    self.selection_clear()
    if self.selected is None: return
    pos = self.get()
    if self.edit_callback():
      item = self.get(pos)
      self.deleteItem()
      # if pos < self.length():
      #   pos += 1
      self.insertItem(item, pos)
      self.set(pos)
  

class TabFrame(Frame):
  def __init__(self, parent, **kwargs):
    Frame.__init__(self, parent, **kwargs)
    self.tabbar = Frame(self)
    self.tabbar.pack(side=TOP, fill=X, expand=0, anchor=N)
    self.tabboard = Frame(self, border=2, relief=RAISED)
    self.tabboard.pack(side=TOP, anchor=N, fill=BOTH, expand=1)
    self.tabs = {}
    self.selected = None
  
  def add(self, name, tab_body):
    tab_button = Label(self.tabbar, text=name, relief=SUNKEN, border=2)
    tab_button.pack(side=LEFT, fill=X, expand=1)
    tab_button.bind("<ButtonRelease-1>", lambda x, w=self, t=name: w.select(t))
    self.tabs[name] = (tab_button, tab_body)
  
  def select(self, name):
    if name not in self.tabs.keys(): return
    if self.selected != None:
      self.tabs[self.selected][0].config(relief=SUNKEN)
      self.tabs[self.selected][1].pack_forget()
    self.tabs[name][0].config(relief=RAISED)
    self.tabs[name][1].pack(padx=5, pady=5)
    self.selected = name
    self.tabs[name][1].focus_set()
  

class Field(Frame):
  def __init__(self, parent, label, value, **kwargs):
    Frame.__init__(self, parent, **kwargs)
    self.parent = parent
    if label:
      self.label = Label(self, text=label, justify=RIGHT, anchor=E)
      self.label.pack(side=LEFT, fill=X, expand=1)
    self.ini_val = value
    self.change_callback = lambda x=None: None
  
  def reset(self):
    self.set(self.ini_val)
  
  def set(self, value):
    pass # mthode abstraite
  
  def get(self):
    pass # mthode abstraite
  
  def bind_onchange(self, callback):
    self.change_callback = callback
  
class TextField(Field):
  def __init__(self, parent, label='', value='', width=30, **kwargs):
    Field.__init__(self, parent, label, value, **kwargs)
    self.field = Entry(self, width=width)
    self.field.insert(END, value)
    self.field.pack(side=LEFT)
    self.focus_set = self.field.focus_set
  
  def bind_onchange(self, callback):
    self.change_callback = callback
    self.field.bind("<FocusOut>", callback)
  
  def get(self):
    return self.field.get()
  
  def set(self, value):
    self.field.delete(0, END)
    self.field.insert(END, value)

class FileField(Field):
  def __init__(self, parent, label='', value='', **kwargs):
    Field.__init__(self, parent, label, value, **kwargs)
    self.field = Entry(self, width=30)
    self.field.insert(END, value)
    self.field.pack(side=LEFT)
    self.but = Button(self, text="...", font=BBUT_FONT, command=self.select_file)
    self.but.pack(side=LEFT)
    self.focus_set = self.field.focus_set
  
  def bind_onchange(self, callback):
    self.change_callback = callback
    self.field.bind("<FocusOut>", callback)
  
  def get(self):
    return self.field.get()
  
  def set(self, value):
    self.field.delete(0, END)
    self.field.insert(END, value)
  
  def select_file(self):
    filename = tkFileDialog.askopenfilename()
    if filename:
      self.set(filename)
      self.change_callback()
  
class IntField(Field):
  def __init__(self, parent, label='', value=0, **kwargs):
    Field.__init__(self, parent, label, value, **kwargs)
    self.field = Entry(self, width=4)
    self.field.insert(END, str(value))
    self.field.pack(side=LEFT)
    self.focus_set = self.field.focus_set
  
  def bind_onchange(self, callback):
    self.change_callback = callback
    self.field.bind("<FocusOut>", callback)
  
  def get(self):
    try:
      return int(self.field.get())
    except:
      return 0
  
  def set(self, value):
    self.field.delete(0, END)
    self.field.insert(END, str(value))
  
class BoolField(Field):
  def __init__(self, parent, label='', value=0, **kwargs):
    Field.__init__(self, parent, '', value, **kwargs)
    self.var = IntVar()
    self.field = Checkbutton(self, text=label, variable=self.var, command=self.onchange, justify=LEFT, anchor=W, **kwargs)
    self.field.pack(side=LEFT)
    self.set(value)
    self.focus_set = self.field.focus_set
  
  def bind_onchange(self, callback):
    self.change_callback = callback
  
  def onchange(self, event=None):
    self.change_callback()
  
  def get(self):
    return self.var.get()
  
  def set(self, value):
    if value: self.field.select()
    else: self.field.deselect()
  
class FontField(Frame):
  def __init__(self, parent, value=None, **kwargs):
    Frame.__init__(self, parent, **kwargs)
    self.parent = parent
    if not value:
      value = ('', '', '')
    self.ini_val = value
    self.police = TextField(self, "Police", value[0])
    self.police.pack(side=TOP, anchor=W)
    fr = Frame(self)
    self.bold = BoolField(fr, "Gras", "bold" in value[2])
    self.bold.pack(side=LEFT)
    self.italic = BoolField(fr, "Italique", "italic" in value[2])
    self.italic.pack(side=LEFT)
    self.size = IntField(fr, "Taille", value[1])
    self.size.pack(side=RIGHT, fill=X, anchor=E)
    fr.pack(side=TOP, pady=1, fill=X, expand=1)
    self.field = self.police
    self.focus_set = self.field.focus_set
  
  def bind_onchange(self, callback):
    self.police.bind_onchange(callback)
    self.size.bind_onchange(callback)
    self.bold.bind_onchange(callback)
    self.italic.bind_onchange(callback)
  
  def get(self):
    police = self.police.get()
    if not police: return ""
    bi = ""
    if self.bold.get(): bi += "bold "
    if self.italic.get(): bi += "italic "
    return (police, self.size.get(), bi[:-1])
  
  def set(self, value):
    self.police.set(value[0])
    self.size.set(value[1])
    self.bold.set("bold" in value[2])
    self.italic.set("italic" in value[2])
  
  def reset(self):
    self.set(self.ini_val)
  
class MultiChoiceField(Field):
  def __init__(self, parent, label='', items=[], value=[], maxbycol=8, **kwargs):
    Field.__init__(self, parent, label, value, **kwargs)
    self.label.pack_forget()
    self.label.config(justify=LEFT)
    self.label.pack(side=TOP, anchor=W, expand=1, fill=X)
    self.fr = Frame(self, **kwargs)
    self.fr.config(border=1, relief=RIDGE)
    self.fr.pack(side=BOTTOM, fill=BOTH, expand=1)
    self.field = {}
    self.items = items
    # self.items.sort()
    for i, item in enumerate(self.items):
      if (i % maxbycol) == 0:
        f = Frame(self.fr, **kwargs)
        f.pack(side=LEFT, anchor=NW, fill=BOTH, expand=1)
      self.field[item] = BoolField(f, item, value=(item in self.ini_val))
      self.field[item].pack(anchor=W, pady=2)
  
  def set(self, value):
    for item in self.items:
      self.field[item].set((item in value))
  
  def get(self):
    return [ item for item in self.items if self.field[item].get() ]
  
class ComboField(Field):
  def __init__(self, parent, label='', items={}, value='', **kwargs):
    Field.__init__(self, parent, label, value, **kwargs)
    self.field = Pmw.OptionMenu(self, command=self.onchange)
    self.setitems(items)
    self.set(value)
    self.field.pack(side=LEFT)
    self.focus_set = self.field.focus_set
  
  def setitems(self, items):
    if type(items) != type({}):
      tmp_items = {}
      for i in items: tmp_items[i] = i
      items = tmp_items
    self.list = items
    self.field.setitems(self.list.values())
  
  def bind_onchange(self, callback):
    self.change_callback = callback
  
  def onchange(self, event=None):
    self.change_callback()
  
  def get(self):
    descr = self.field.getvalue()
    for key in self.list.keys():
      if self.list[key] == descr:
        return key
  
  def set(self, value):
    try:
      self.field.setvalue(self.list[value])
    except KeyError:
      pass
  
class ColorField(Field):
  def __init__(self, parent, label='', value='', **kwargs):
    Field.__init__(self, parent, label, value, **kwargs)
    self.field = Entry(self, width=8)
    self.field.pack(side=LEFT)
    self.field.insert(END, value)
    self.but = Button(self, text=" "*12, font=BBUT_FONT, command=self.select_color)
    if value: self.but.config(bg=value)
    self.but.pack(side=LEFT)
    self.field.bind("<FocusOut>", self.update_color)
    self.focus_set = self.field.focus_set
  
  def bind_onchange(self, callback):
    self.change_callback = callback
    self.field.bind("<FocusOut>", callback, "+")
  
  def get(self):
    return self.field.get()
  
  def set(self, value):
    self.field.delete(0, END)
    self.field.insert(END, value)
    self.update_color()
  
  def select_color(self):
    try:
      self.but.focus_set()
    except FocusException:
      return
    rgb, color = tkColorChooser.askcolor(self.ini_val, parent=self.parent)
    if rgb != None:
      self.field.delete(0, END)
      self.field.insert(END, "#%02x%02x%02x" % rgb)
      self.but.config(bg="#%02x%02x%02x" % rgb)
      self.change_callback()
  
  def update_color(self, event=None):
    value = self.get()
    if not value:
      return
    try:
      self.but.config(bg=value)
    except:
      log.warning("ColorField: Couleur invalide: %s" % value)
  

class Combo:
  def __init__(self, parent, items={}, value=None):
    self.var = StringVar()
    if type(items) != type({}):
      tmp_items = {}
      for i in items: tmp_items[i] = i
      items = tmp_items
    self.list = items
    self.field = apply(OptionMenu, [parent, self.var]+[self.list[key] for key in self.list.keys()])
    self.field.config(anchor=W, justify=LEFT)
    if value != None:
      self.set(value)
    self.pack = self.field.pack
    self.focus_set = self.field.focus_set
  
  def get(self):
    descr = self.var.get()
    for key in self.list.keys():
      if self.list[key] == descr:
        return key
  
  def set(self, value):
    try:
      self.var.set(self.list[value])
    except KeyError:
      pass
  

#-------------------------------------------------------------------------
# Programme de taiste
class Meuh:
  def __init__(self, id):
    self.id = id
    self.label = "Toto%s" % id
    self.image = None

def kikoo():
  global lbl, tree
  lbl.config(text="Selected: %s" % tree.selected)

if __name__ == '__main__':
  # contruct some items:
  root = Tk()
  img_path = os.path.join('plugins', 'smileys')
  imgs = [ PhotoImage(file=os.path.join(img_path, "bunny.gif")),
           PhotoImage(file=os.path.join(img_path, "chief.gif")),
           PhotoImage(file=os.path.join(img_path, "cool.gif")),
           PhotoImage(file=os.path.join(img_path, "deal.gif")),
           PhotoImage(file=os.path.join(img_path, "hase.gif")),
           PhotoImage(file=os.path.join(img_path, "idee.gif")),
           PhotoImage(file=os.path.join(img_path, "ka.gif")),
           PhotoImage(file=os.path.join(img_path, "kiss.gif")),
           PhotoImage(file=os.path.join(img_path, "bunny2.gif")),
           PhotoImage(file=os.path.join(img_path, "devil.gif"))
          ]
  t = {}
  for i in "0123456789":
    t[i] = TreeStruct(Meuh(i))
    t[i].root.image = imgs[int(i)]
  t["0"].childs = [ t["1"], t["2"], t["3"] ]
  t["4"].childs = [ t["5"], t["6"] ]
  t["5"].childs = [ t["7"], t["8"], t["9"] ]
  stree = TreeStruct(Meuh("root"), [ t["0"], t["4"] ])
  stree.collapseAll()
  
  #allonzy
  tree = Tree(root, stree, width=20, height=8)
  tree.pack(expand=1, fill="both")
  tree.select("2")
  tree.config(background='#DAFCFF')
  lbl = Label(root, text="")
  lbl.pack()
  tree.bind_select(kikoo)
  root.mainloop()
