# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module rssreader
##   Dfinition du module de bouchot RSSReader :
##   monitoring d'un feed rss (pour les news, etc.)
## ----------------------------------------------------------------------

import logging
log = logging.getLogger('pycc')

import sys, os, os.path, md5
sys.path.append('../..')

import xml.dom.minidom

from constants import *
from modules.module import *
from coinlib.persistants import OptionSet, Option
from coinlib.utils import *
from coinlib.parsers import *

def build_module(parent, name):
  return RSSReader(parent, name)

import re
regexp_newsid = re.compile("^.*/([0-9]+)/([0-9]+)/([0-9]+)/([^/,.]+)[,.][^/]*$")

class RSSReaderOptions(OptionSet):
  """ Options du module RSSReader
  """
  def setObjects(self):
    self._objects = {
      'descr':        Option("Description (nom pour affichage)", "RSSReader"),
      'short_name':   Option("Nom court (pour la diode)", "N"),
      'backend':      Option("URL du fichier de backend", ""),
      'docked':       Option("Dock dans l'onglet", 1, TYPE_BOOL),
      'encoding':     Option("Encoding du backend", "iso-8859-15"),
      'check_delay':  Option("Nombre de secondes entre 2 checks du backend", 300, TYPE_INT),
      'max_history':  Option("Nombre maximum d'lments  conserver", 50, TYPE_INT),
      'read_at_exp':  Option("Marquer lu ds l'ouverture dans le viewer", 0, TYPE_BOOL),
      'slip_crade':   Option("Slip pas propre", 0, TYPE_BOOL),
      'keep_cr':      Option("Conserver les retours  la ligne", 1, TYPE_BOOL),
      'tags_encoded': Option("Tags encods", 0, TYPE_BOOL),
      'find_author':  Option("Trouver l'auteur dans l'url (journaux dlfp)", 0, TYPE_BOOL),
      'find_date':    Option("Trouver la date dans l'url (news dlfp)", 0, TYPE_BOOL),
      'find_id':      Option("Trouver l'ID dans l'url", 0, TYPE_BOOL),
      'display_img':  Option("Dtecter et afficher les images", 1, TYPE_BOOL),
      'toedit':       Option("Editables", { \
          "Gnral": [ 'descr', 'short_name', 'backend', 'check_delay', 'max_history', 'read_at_exp', 'docked' ],
          "Parsing": [ 'tags_encoded', 'keep_cr', 'slip_crade', 'find_date', 'find_author', 'find_id', 'display_img', 'encoding' ]
        } )
    }


class RSSItem(object):
  
  def __init__(self, attrs={}):
    self.attributes = attrs
    if attrs.has_key('id'):
      self.id = attrs['id']
    else:
      self.computeId()
    self.read     = 0   # Flag lu/nonlu
    self.expanded = 0   # Flag description ouverte/ferme
    self.next = None    # Pointeur sur le suivant
  
  def computeId(self):
    self.id = md5.new(str(self.attributes)).hexdigest()
  
  def getTitle(self):
    if not self.attributes.has_key('title'):
      return u'[pas de titre]'
    return self.attributes['title']
  
  def getLink(self):
    for lkey in ['link', 'guid']:
      if self.attributes.has_key(lkey):
        return self.attributes[lkey]
      else:
        continue
    return ''
  
  def getDate(self):
    for dkey in ['date', 'pubDate', 'dc:date']:
      if self.attributes.has_key(dkey):
        return self.attributes[dkey]
      else:
        continue
    return ''
  
  def getAuthor(self):
    for akey in ['author', 'creator', 'dc:creator']:
      if self.attributes.has_key(akey):
        return self.attributes[akey]
      else:
        continue
    return ''
  
  def getDescr(self):
    try:
      return self.attributes['description']
    except:
      return ''
  
  def markRead(self):
    self.read = 1
  
  def markUnread(self):
    self.read = 0
  

class RSSViewer(GenericViewer):

  def insertItem(self, item):
    self.addFoldSign(item)
    self.list.config(state=NORMAL)
    self.list.insert(INSERT, item.getDate(), "time")
    self.list.insert(INSERT, " ")
    self.list.insert(INSERT, item.getAuthor(), "login")
    self.list.insert(INSERT, " ")
    t = "id%s" % item.id
    self.list.insert(INSERT, item.getTitle(), t)
    self.list.insert(INSERT, "\n")
    self.setStyleTag(t, "a")
    self.setStyleTag(t, "url_"+item.getLink())
    self.module.bouchot.pycc.addUrlInList(item.getLink(), item.getDate()+"@"+self.module.name)
    if item.expanded:
      text = toUnicode(item.getDescr(), self.module.options.val('encoding'))
      msg, tags = self.module.parseDescr(text)
      # self.module.bouchot.pycc.addUrls([tag[4:] for tag in tags if tag[:4]=="url_"], item.getDate()+"@"+self.module.name)
      start = self.list.index("insert")
      self.list.insert(INSERT, msg+"\n\n", 'descr')
      self.addTags(tags, start)
    self.list.config(state=DISABLED)
    if item.read:
      self.markRead(item)


def getText(nodelist, encoding='iso-8859-15'):
  rc = ""
  for node in nodelist:
    if node.nodeType == node.TEXT_NODE:
      rc = rc + node.data
  return toUnicode(rc, encoding)

def getXML(nodelist, encoding='iso-8859-15'):
  rc = ""
  for node in nodelist:
    rc = rc + node.toxml()
  rc = rc.replace('<![cdata[','')
  rc = rc.replace('<![CDATA[','')
  rc = rc.replace(']]>','')
  return toUnicode(rc, encoding)

def handle_stinking_underpants(text):
  msg = text.replace("&lt;", "<")
  start = msg.find("&")
  end = 0
  res = ""
  while start >= 0:
    res += msg[end:start]
    if msg[start+1] == " ":
      res += "&amp;"
      end = start+1
      start = msg.find("&", end)
      continue
    end = msg.find(";", start)
    if end < start:
      end = start
      break
    entity = msg[start+1:end]
    for c in entity:
      if c.lower() not in "azertyuiopqsdfghjklmwxcvbn1234567890_-#":
        entity = "amp;%s" % entity
        break
    res += "&%s;" % entity
    end += 1
    start = msg.find("&", end)
  res += msg[end:]
  return res


class RSSReader(Module):
  
  def init(self):
    self.lastId = None
    self.firstId = None
    self.title = self.options.val('descr')
    self.rssElements = ['title', 'link', 'guid', 'description', 'pubDate', 'date', 'dc:date', 'author', 'creator', 'dc:creator']
    # self.img_urls = {}
  
  def build_viewer(self, parent):
    self.viewer = RSSViewer(parent, self)
  
  def apply_styles(self):
    self.viewer.apply_styles()
  
  def load_config(self):
    self.options = RSSReaderOptions(path=self.config_path)
  
  def unload(self):
    # AFR ?
    pass
  
  def parse(self, file):
    tmplist = []
    text = file.read()
    if self.options.val('slip_crade'):
      text = handle_stinking_underpants(text)
    encoding = self.getEncoding(text)
    try:
      dom = xml.dom.minidom.parseString(text)
    except:
      self.error = "Erreur de parsing XML (fichier rss moisi ?)"
      return []
    channel = dom.getElementsByTagName("channel")[0]
    self.title = getText(channel.getElementsByTagName("title")[0].childNodes, encoding)
    if not self.options.val('keep_cr'):
      self.title = self.title.replace('\n', ' ')
    for item in dom.getElementsByTagName("item"):
      attrs = {}
      for elem in self.rssElements:
        domlist = item.getElementsByTagName(elem)
        if len(domlist):
          if elem == 'description' or elem == 'title':
            msg = getXML(domlist[0].childNodes, encoding)
            attrs[elem] = msg
          else:
            attrs[elem] = getText(domlist[0].childNodes, encoding)
            if elem == 'link':
              link = attrs[elem]
              if self.options.val('find_date'):
                newsid = regexp_newsid.match(link)
                if self.options.val('find_id'):
                  attrs['id'] = newsid.groups()[3]
                attrs['date'] = "%s/%s/%s" % (newsid.groups()[2], newsid.groups()[1], newsid.groups()[0])
              if self.options.val('find_author'):
                pos1 = link.find("~")
                pos2 = link.find("/", pos1)
                attrs['author'] = link[pos1+1:pos2]
                if self.options.val('find_id'):
                  attrs['id'] = link[pos2+1:-5]
          if not self.options.val('keep_cr'):
            attrs[elem] = attrs[elem].replace('\n', ' ')
      newitem = RSSItem(attrs)
      newitem.read = self.isRead(newitem.id)
      tmplist.append(newitem)
    dom.unlink()
    tmplist.reverse()
    return tmplist
  
  def process(self, items=[]):
    for rssitem in items:
      if self.firstId is None:
        self.firstId = rssitem.id
      if rssitem.id not in self.objects.keys():
        log.debug("%s: nouvel rssitem: %s" % (self.name, rssitem.id))
        self.objects[rssitem.id] = rssitem
        if self.lastId is not None:
          self.objects[self.lastId].next = rssitem
        self.lastId = rssitem.id
        if self.viewer_open:
          self.viewer.insertItem(rssitem)
        self.bouchot.setState(STATE_NEW)
    self.updateLed()
  
  def updateLed(self):
    self.led.setText(str(len([i for i in self.objects.values() if not i.read])))
  
  def markRead(self, item):
    if not item.read:
      try:
        f = open(os.path.join(self.config_path, "read"), 'a')
        f.write(item.id+"\n")
        f.close()
      except:
        log.warning("%s: Ecriture dans le fichier 'read' impossible" % self.name)
    item.read = 1
    if self.viewer_open:
      self.viewer.markRead(item)
    self.updateLed()
  
  def isRead(self, id):
    try:
      f = open(os.path.join(self.config_path, "read"), 'r')
      readlist = f.read().split("\n")
      f.close()
    except:
      log.warning("%s: Lecture dans le fichier 'read' impossible" % self.name)
      return 0
    return id in readlist
  
  def parseDescr(self, text):
    if self.options.val('tags_encoded'):
      slip_style = "Tags encoded"
    else:
      slip_style = "Tags not encoded"
    text = htmlParser.pre_parse(text, slip_style, self.options.val('encoding'))
    if self.options.val('display_img'):
      start = text.find("<img")
      end = text.find(">", start)
      while start > -1 and end > start:
        url = getAttr("src", text[start:end])
        text = text[:start] + "<img>[image:" + url + "]</img>" + text[end+1:]
        start = text.find("<img", start+5)
        end = text.find(">", start)
        self.bouchot.pycc.getImageWithCache(url)
    msg, tags = htmlParser.html_parse(text, encoding=self.options.val('encoding'))
    return (msg, tags)
  