# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Plug-in Plateau de Fruits de Mer
## ----------------------------------------------------------------------

import logging
log = logging.getLogger('pycc')
import sys
sys.path.append('..')
from coinlib.plugin import *
from coinlib.persistants import Option, OptionSet
from cointklib.widgets import OptionsEditor
from cointklib.dialogs import CoinDialog
from cointklib.basicwidgets import *

from time import *
import os, os.path

def build_plugins(pycc):
  return { \
    'pdfm': Pdfm(pycc, "Plateau de fruits de mer", "Sauvegarde la tribune courante dans un fichier HTML")
  }

class PdfmOptions(OptionSet):
  
  def setName(self, name):
    self.name = "Pdfm"
  
  def setObjects(self):
    self._objects = {  
      'shots_path': Option("Sauver dans", value=""),
      'file_name': Option("Fichier scrinechote", value="%t_%Y-%m-%d-%H%M%S.html"),
      'default_css': Option("Style par dfaut", value="llfr", type=TYPE_CHOICE),
      'toedit': Option("Editables", {"Gnral": ['shots_path', 'file_name', 'default_css']})
    }

class PdfmEditor(OptionsEditor):
  
  def disp_choice(self, fr, name, option):
    self.widgets[name] = ComboField(fr, option.descr, self.extra_arg.csslist.keys(), value=option.get())
    self.widgets[name].pack(pady=2, fill=X)

class PdfmDialog(CoinDialog):
  
  def __init__(self, title, parent, object, extra_arg):
    self.object = object
    self.extra_arg = extra_arg
    CoinDialog.__init__(self, parent, title, [BUT_OK, BUT_CANCEL], modal=1)
  
  def body(self, master):
    self.w_options = PdfmEditor(master, self.object, extra_arg=self.extra_arg)
    self.w_options.pack()
  
  def apply(self):
    self.w_options.apply()

class Pdfm(Plugin):
  
  def load_config(self):
    self.options = PdfmOptions(path=self.config_path)
    if not self.options.val('shots_path'):
      self.options.set('shots_path', os.path.join(self.pycc.config_path, "shots"))
  
  def config_frame(self, master):
    self.get_css()
    return PdfmEditor(master, self.options, self)
  
  def action_menu(self):
    self.get_css()
    self.pycc.root.canceled = 1
    PdfmDialog("Scrinechote Tribune", self.pycc.root, self.options, self)
    if not self.pycc.root.canceled:
      self.process(self.pycc.bouchots[self.pycc.get_bouchot()])
  
  def get_css(self):
    css_path = os.path.join(self.pycc.script_path, "config", "shots")
    shots_path = os.path.join(self.pycc.config_path, "shots")
    if not os.path.exists(shots_path):
      os.mkdir(shots_path)
    self.csslist = {}
    for file in os.listdir(css_path):
      if file[-4:] == '.css':
        self.csslist[file[:-4]] = "file://"+os.path.join(css_path, file)
    for file in os.listdir(shots_path):
      if file[-4:] == '.css':
        self.csslist[file[:-4]] = "file://"+os.path.join(shots_path, file)
  
  def process(self, bouchot, context=PINNI, post=None):
    if bouchot.trib is None: return
    if not bouchot.trib.loaded: return
    tstamp = gmtime(time())
    datetime = strftime("%d/%m/%Y %H:%M:%S")
    self.get_css()
    html_css = u''
    for cssname, cssfile in self.csslist.items():
      if cssname == self.options.val('default_css'):
        alt = ""
      else:
        alt = "alternate "
      html_css += u'    <link rel="%sstylesheet" title="%s" media="screen" type="text/css" href="%s" />\n' % (alt, cssname, cssfile)
    pdfm_file = os.path.join(self.options.val('shots_path'), strftime(self.options.val('file_name').replace("%t", bouchot.name), tstamp))
    title = "Tribune %s - Plateau de fruits de mer" % (bouchot.name)
    encoding = bouchot.trib.options.val('encoding')
    
    html_head = u"""<?xml version="1.0" encoding="%s"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "DTD/xhtml1-strict.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="fr" lang="fr">
  <head>
    <title>%s</title>
""" % (encoding, title)
    
    html_body = u"""  </head>
  <body>
    <script language="javascript" type="text/javascript">
      function showref(id) {
        post = document.getElementById(id);
        post.oldstyle = post.className;
        post.className += " highlight";
        var spans = document.getElementsByTagName("span");
        for (var i = 0; i < spans.length; i++) {
          if (spans[i].className == "horloge_ref") {
            if (spans[i].title == ("ref_"+id)) {
              spans[i].oldstyle = spans[i].className;
              spans[i].className += " highlight";
            }
          }
        }
      }
      function clearref(id) {
        post = document.getElementById(id);
        post.className = post.oldstyle;
        var spans = document.getElementsByTagName("span");
        for (var i = 0; i < spans.length; i++) {
          if (spans[i].title == ("ref_"+id)) {
            spans[i].className = "horloge_ref";
          }
        }
      }
    </script>
    <h1>%s</h1>
    <ul>
  """ % (title)
    
    html_foot = u"""  </ul>
    <p class="footer">Pondu le %s</p>
  </body>
</html>""" % (datetime)
    
    f = open(pdfm_file, "w")
    f.write((html_head+html_css+html_body).encode(encoding))
    
    # On recre la liste des posts car si on utilisait directement trib.posts, le tableau
    # risque de changer pendant la boucle for (vu qu'a priori le thread de polling de
    # la tribune continue de tourner pendant l'excution du plugin)
    postlist = [post for id, post in bouchot.trib.posts.items()]
    postlist.sort()
    
    cpt_post = 0
    for post in postlist:
      message = ""
      cpt = 0
      for ch in post.message+" ":
        for tag in ['a', 'b', 's', 'u', 'i', 'tt', 'clock']:
          if tag in post.tags:
            for begin, end in post.tags[tag]:
              if begin == cpt:
                if tag == 'clock':
                  message += '<span class="horloge_ref"'
                elif tag == "a":
                  for imgtag in post.tags.keys():
                    if imgtag[:7] == "smiley_":
                      for mark in post.tags[imgtag]:
                        if mark[0] == begin:
                          message += '<img src="http://%s/%s.gif" />' % (bouchot.trib.options.val('smileys_server'), imgtag[7:])
                  message += "<"+tag
                  for urltag in post.tags.keys():
                    if urltag[:4] == "url_":
                      for mark in post.tags[urltag]:
                        if mark[0] == begin:
                          message += ' href="%s"' % urltag[4:]
                else:
                  message += "<"+tag
                if tag == "clock":
                  for reftag in post.tags.keys():
                    if reftag not in ['a', 'b', 's', 'u', 'i', 'tt', 'clock'] and reftag[:4] != "url_":
                      for mark in post.tags[reftag]:
                        if mark[0] == begin:
                          message += ' title="ref_%s" onmouseover="showref(\'%s\')" onmouseout="clearref(\'%s\')"' % (reftag[3:9], reftag[3:9], reftag[3:9])
                message += ">"
        for tag in ['clock', 'tt', 'i', 'u', 's', 'b', 'a']:
          if tag in post.tags:
            for begin, end in post.tags[tag]:
              if end == cpt or cpt >= len(post.message):
                if tag == 'clock':
                  message += "</span>"
                else:
                  message += "</%s>" % tag
        if ch == '<':
          message += "&lt;"
        elif ch == '>':
          message += "&gt;"
        elif ch == '&':
          message += "&amp;"
        else:
          message += ch
        cpt += 1
      cpt_post += 1
      if post.login:
        login = '<span title="%s" class="login">%s</span>' % (post.info, post.login)
      else:
        login = '<span title="%s" class="ua_login">%s</span>' % (post.info, post.info[:20])
      id = "".join(post.disp('time').split(":"))
      html_curpost = '<li id="%s"><span class="horloge" title="%s" onmouseover="showref(\'%s\')" onmouseout="clearref(\'%s\')">%s</span> %s %s</li>\n' % (id, post.id, id, id, post.disp('time'), login, message)
      f.write(html_curpost.encode(encoding))
    
    f.write(html_foot.encode(encoding))
    f.close()
    
    bouchot.pycc.setStatus("Plateau de Fruits de Mer sauvegard dans %s" % pdfm_file)
  
