# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module stringutils
##   Ce module dfinit 7 plug-ins lmentaires pour transformer du texte
## ----------------------------------------------------------------------

import logging
log = logging.getLogger('pycc')
import sys
sys.path.append('..')
from coinlib.plugin import *

import string
from random import randrange

def build_plugins(pycc):
  return { \
    'uppercase': Uppercase(pycc, "UPPERCASE", "Passe le texte en MAJUSCULES"),
    'lowercase': Lowercase(pycc, "lowercase", "Passe le texte en minuscules"),
    'mixedcase': Mixedcase(pycc, "mIxEdcASe", "Alterne alatoirement majuscules et minuscules dans le texte"),
    'capwords':  CapitalizeWords(pycc, "CapitalizeWords", "Met les premires lettres de chaque mot du texte en majuscule"),
    'wikifier':  Wikifier(pycc, "Wikifier", "Met tous les mots du texte en [Style] [Wiki]"),
    'trevni':    Trevni(pycc, "Inverser", "Retourne le texte (criture  l'envers)"),
    'rot13':     Rot13(pycc, "Rot-13", "Encode/dcode le texte en rot13")
  }


class Uppercase(TextPlugin):
  
  def text_transform(self, text, tags={}):
    return text.upper()
  
class Lowercase(TextPlugin):
  
  def text_transform(self, text, tags={}):
    return text.lower()
  
class Mixedcase(TextPlugin):
  
  def text_transform(self, text, tags={}):
    res = ''
    for ch in text:
      if randrange(0, 2):
        ch = ch.upper()
      else:
        ch = ch.lower()
      res += ch
    return res

class CapitalizeWords(TextPlugin):
  
  def text_transform(self, text, tags={}):
    return string.capwords(text)

class Wikifier(TextPlugin):
  
  def text_transform(self, text, tags={}):
   return " ".join(["["+word.capitalize()+"]" for word in text.split()])

class Trevni(TextPlugin):
  
  def text_transform(self, text, tags={}):
    res = ''
    for ch in text:
      res = ch + res
    return res

class Rot13(TextPlugin):
  
  def text_transform(self, text, tags={}):
    res = ''
    for ch in text:
      ascode = ord(ch)
      if (ascode > 64 and ascode < 78) or (ascode > 96 and ascode < 110):
        res = res + chr(ascode+13)
      elif (ascode > 77 and ascode < 91) or (ascode > 109 and ascode < 123):
        res = res + chr(ascode-13)
      else:
        res = res + ch
    return res
